import sys

import scipy.io

from .__about__ import __version__
from .main import show, write_png


def _get_version_text():
    return "\n".join(
        [
            "betterspy {} [Python {}.{}.{}]".format(
                __version__,
                sys.version_info.major,
                sys.version_info.minor,
                sys.version_info.micro,
            ),
            "Copyright (c) 2018-2020 Nico Schlömer <nico.schloemer@gmail.com>",
        ]
    )


def main(argv=None):
    # Parse command line arguments.
    parser = _get_parser()
    args = parser.parse_args(argv)

    A = scipy.io.mmread(args.infile)
    if args.outfile is None:
        show(A)
    else:
        write_png(args.outfile, A, args.border_width, args.border_color, args.colormap)

    return


def _get_parser():
    import argparse

    parser = argparse.ArgumentParser(
        description=("Show sparsity structure of matrix"),
        formatter_class=argparse.RawTextHelpFormatter,
    )

    parser.add_argument("infile", type=str, help="input matrix market file")

    parser.add_argument(
        "outfile", type=str, nargs="?", default=None, help="output png file (optional)"
    )

    parser.add_argument(
        "--border-width",
        "-w",
        required=False,
        type=int,
        default=0,
        help="border width (default: 0)",
    )

    parser.add_argument(
        "--border-color",
        "-b",
        required=False,
        type=str,
        default="0.5",
        help="border color (default: 0.5, gray)",
    )

    parser.add_argument(
        "--colormap",
        "-c",
        required=False,
        type=str,
        default=None,
        help="border color (default: 0.5, gray)",
    )

    parser.add_argument(
        "--version",
        "-v",
        action="version",
        version=_get_version_text(),
        help="display version information",
    )
    return parser



import tempfile
from pathlib import Path

import imageio
import numpy
import pytest
from scipy import sparse

import betterspy


def test_show():
    M = sparse.rand(20, 20, density=0.1)
    betterspy.show(M)


@pytest.mark.parametrize(
    "ref, kwargs",
    [
        (6875310, {}),
        (7524085, {"border_width": 1}),
        (21306270, {"border_width": 1, "border_color": "red"}),
        (4981037, {"colormap": "viridis"}),
        (7101351, {"colormap": "viridis", "border_width": 1}),
    ],
)
def test_png(ref, kwargs):
    M = sparse.rand(20, 30, density=0.1, random_state=123)
    numpy.random.seed(123)

    with tempfile.TemporaryDirectory() as temp_dir:
        filepath = Path(temp_dir) / "test.png"
        betterspy.write_png(filepath, M, **kwargs)
        im = imageio.imread(filepath)
        y = numpy.random.randint(0, 100, size=numpy.prod(im.shape))
        assert numpy.dot(y, im.flatten()) == ref


def test_readme_images():
    pytest.importorskip("dolfin")
    import meshzoo
    from dolfin import (
        EigenMatrix,
        FunctionSpace,
        Mesh,
        MeshEditor,
        TestFunction,
        TrialFunction,
        assemble,
        dot,
        dx,
        grad,
    )

    points, cells = meshzoo.rectangle(-1.0, 1.0, -1.0, 1.0, 20, 20)

    # Convert points, cells to dolfin mesh
    editor = MeshEditor()
    mesh = Mesh()
    # topological and geometrical dimension 2
    editor.open(mesh, "triangle", 2, 2, 1)
    editor.init_vertices(len(points))
    editor.init_cells(len(cells))
    for k, point in enumerate(points):
        editor.add_vertex(k, point[:2])
    for k, cell in enumerate(cells.astype(numpy.uintp)):
        editor.add_cell(k, cell)
    editor.close()

    V = FunctionSpace(mesh, "CG", 1)
    u = TrialFunction(V)
    v = TestFunction(V)
    L = EigenMatrix()
    assemble(dot(grad(u), grad(v)) * dx, tensor=L)
    A = L.sparray()

    # M = A.T.dot(A)
    M = A

    with tempfile.TemporaryDirectory() as temp_dir:
        filepath = Path(temp_dir) / "test.png"
        betterspy.write_png(filepath, M, border_width=2)

    # betterspy.write_png(
    #     'ATA.png', M, border_width=2,
    #     colormap='viridis'
    #     )


def test_cli():
    this_dir = Path(__file__).resolve().parent
    mmfile = this_dir / "data" / "gre_343_343_crg.mm"
    betterspy.cli.main([mmfile.as_posix()])
    betterspy.cli.main([mmfile.as_posix(), "out.png"])


if __name__ == "__main__":
    test_readme_images()



import pathlib

# conda-forge doesn't have exdown
# exdown = pytest.importorskip("exdown")
import exdown

this_dir = pathlib.Path(__file__).resolve().parent

test_readme = exdown.pytests(this_dir / ".." / "README.md", syntax_filter="python")
